﻿//---------------------------------------------------------------------------
//
//   StationSpy
//
//          Copyright(C) 2006-2012, tetraface Inc.
//
//     This plug-in watches messages sent to a station plug-in. The sent 
//    messages are displayed in the list. Please use to confirm what and 
//    when messages are sent.
//     WTL (Windows Template Library) is required separately for a build.
//
//    　常駐プラグインに送られるメッセージを監視します。送られたメッセー
//    ジはリストに一覧表示されます。
//    　常駐プラグインにどういったタイミングでメッセージが送られるかを確
//    認する目的でご利用ください。
//    　ビルドにはWTL(Windows Template Library)が別途必要です。
//
//---------------------------------------------------------------------------
#include "stdafx.h"
#include "resource.h"
#include "maindlg.h"
#include "StationSpyPlugin.h"

static StationSpyPlugin plugin;


// Constructor
// コンストラクタ
StationSpyPlugin::StationSpyPlugin()
{
	dlgMain = NULL;
	m_bActivate = false;
	m_bOnDraw = false;
	m_bOnUpdateScene = false;
	m_bOnUpdateUndo = false;
}

// Destructor
// デストラクタ
StationSpyPlugin::~StationSpyPlugin()
{
	if (dlgMain != NULL){
		// Also deletes the window here just to make sure though it has been 
		// deleted in Exit().
		// Exit()で破棄されるはずだが一応念のためここでも
		dlgMain->DestroyWindow();
		delete dlgMain;
		dlgMain = NULL;
	}
}

//---------------------------------------------------------------------------
//  GetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
void StationSpyPlugin::GetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0x9CE001E3;
}

//---------------------------------------------------------------------------
//  GetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
const char *StationSpyPlugin::GetPlugInName(void)
{
	return "Station Spy       Copyright(C) 2006-2012, tetraface Inc.";
}

//---------------------------------------------------------------------------
//  EnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
const char *StationSpyPlugin::EnumString(void)
{
	return "Station Spy";
}


//---------------------------------------------------------------------------
//  Initialize
//    アプリケーションの初期化
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::Initialize()
{
	if (dlgMain == NULL){
		// Assign a handle of the main window to the application.
		// アプリケーションにメインウインドウのハンドルを割り当て
		dlgMain = new CMainDlg(this);
		if(dlgMain->Create(MQ_GetWindowHandle()) == NULL)
		{
			delete dlgMain;
			dlgMain = NULL;
			ATLTRACE(_T("Main dialog creation failed!\n"));
			return 0;
		}

		// Always place the window above.
		// ウインドウを常に前面に
		dlgMain->SetWindowPos(HWND_TOPMOST, 0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);

		dlgMain->AddMessage("Initialize");
	}
	return TRUE;
}

//---------------------------------------------------------------------------
//  Exit
//    アプリケーションの終了
//---------------------------------------------------------------------------
void StationSpyPlugin::Exit()
{
	if (dlgMain != NULL){
		dlgMain->AddMessage("Exit");
		dlgMain->DestroyWindow();
		delete dlgMain;
		dlgMain = NULL;
	}
}

//---------------------------------------------------------------------------
//  Activate
//    表示・非表示切り替え要求
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::Activate(MQDocument doc, BOOL flag)
{
	if (dlgMain == NULL)
		return FALSE;

	// Switch showing and hiding the window.
	// ウインドウの表示・非表示切り替え
	m_bActivate = flag ? true : false;
	dlgMain->ShowWindow(m_bActivate ? SW_SHOW : SW_HIDE);

	char buf[256];
	sprintf_s(buf, sizeof(buf), "Activate : flag=%s", flag ? "TRUE" : "FALSE");
	dlgMain->AddMessage(buf);

	return m_bActivate;
}

//---------------------------------------------------------------------------
//  IsActivated
//    表示・非表示状態の返答
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::IsActivated(MQDocument doc)
{
	// Return TRUE if a window has been not created yet.
	// ウインドウがまだ生成されていないならTRUE
	if (dlgMain == NULL)
		return FALSE;

	return m_bActivate;
}

//---------------------------------------------------------------------------
//  OnMinimize
//    ウインドウの最小化への返答
//---------------------------------------------------------------------------
void StationSpyPlugin::OnMinimize(MQDocument doc, BOOL flag)
{
	char buf[256];
	sprintf_s(buf, sizeof(buf), "OnMinimize : flag=%s", flag ? "TRUE" : "FALSE");
	dlgMain->AddMessage(buf);
}

//---------------------------------------------------------------------------
//  OnReceiveUserMessage
//    プラグイン独自のメッセージの受け取り
//---------------------------------------------------------------------------
int StationSpyPlugin::OnReceiveUserMessage(MQDocument doc, DWORD src_product, DWORD src_id, const char *description, void *message)
{
	int len = strlen(description);
	char *buf = (char*)malloc(len + 200);
	sprintf_s(buf, sizeof(buf), "OnReceiveUserMessage : src_plugin=[%08x %08x] description=%s", src_product, src_id, description);
	dlgMain->AddMessage(buf);

	// Operate a special action only when the message was sent from SingleMove plug-in.
	// SingleMoveプラグインからの通知のみ、特別な動作を行う
	if (src_product == 0x56A31D20 && src_id == 0x3901EA9B){
		if (strcmp(description, "text") == 0){
			sprintf_s(buf, sizeof(buf), "  from SingleMove : %s", (const char*)message);
			dlgMain->AddMessage(buf);
		}
	}

	free(buf);

	// Return 59 without a deep idea.
	// 適当に59を返している
	return 59;
}

//---------------------------------------------------------------------------
//  OnDraw
//    描画時の処理
//---------------------------------------------------------------------------
void StationSpyPlugin::OnDraw(MQDocument doc, MQScene scene, int width, int height)
{
	if (!m_bOnDraw)
		return;

	char buf[256];
	sprintf_s(buf, sizeof(buf), "OnDraw : width=%d, height=%d", width, height);
	dlgMain->AddMessage(buf);
}


//---------------------------------------------------------------------------
//  OnNewDocument
//    ドキュメント初期化時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnNewDocument(MQDocument doc, const char *filename, NEW_DOCUMENT_PARAM& param)
{
	char buf[256+MAX_PATH];
	sprintf_s(buf, sizeof(buf), "OnNewDocument : filename=%s", (filename!=NULL) ? filename : "(none)");
	dlgMain->AddMessage(buf);
}

//---------------------------------------------------------------------------
//  OnEndDocument
//    ドキュメント終了時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnEndDocument(MQDocument doc)
{
	dlgMain->AddMessage("OnEndDocument");
}

//---------------------------------------------------------------------------
//  OnSaveDocument
//    ドキュメント保存時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnSaveDocument(MQDocument doc, const char *filename, SAVE_DOCUMENT_PARAM& param)
{
	char buf[256+MAX_PATH];
	sprintf_s(buf, sizeof(buf), "OnSaveDocument : filename=%s", (filename!=NULL) ? filename : "(none)");
	dlgMain->AddMessage(buf);
}

//---------------------------------------------------------------------------
//  OnUndo
//    アンドゥ実行時
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::OnUndo(MQDocument doc, int undo_state)
{
	char buf[256];
	sprintf_s(buf, sizeof(buf), "OnUndo : undo_state=%d", undo_state);
	dlgMain->AddMessage(buf);
	
	return FALSE;
}

//---------------------------------------------------------------------------
//  OnRedo
//    リドゥ実行時
//---------------------------------------------------------------------------
BOOL StationSpyPlugin::OnRedo(MQDocument doc, int redo_state)
{
	char buf[256];
	sprintf_s(buf, sizeof(buf), "OnRedo : redo_state=%d", redo_state);
	dlgMain->AddMessage(buf);

	return FALSE;
}

//---------------------------------------------------------------------------
//  OnUpdateUndo
//    アンドゥ状態更新時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnUpdateUndo(MQDocument doc, int undo_state, int undo_size)
{
	if (!m_bOnUpdateUndo)
		return;

	char buf[256];
	sprintf_s(buf, sizeof(buf), "OnUpdateUndo : undo_state=%d undo_size=%d", undo_state, undo_size);
	dlgMain->AddMessage(buf);
}

//---------------------------------------------------------------------------
//  OnObjectModified
//    オブジェクトの編集時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnObjectModified(MQDocument doc)
{
	dlgMain->AddMessage("OnObjectModified");
}

//---------------------------------------------------------------------------
//  OnObjectSelected
//    オブジェクトの選択状態の変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnObjectSelected(MQDocument doc)
{
	dlgMain->AddMessage("OnObjectSelected");
}

//---------------------------------------------------------------------------
//  OnUpdateObjectList
//    カレントオブジェクトの変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnUpdateObjectList(MQDocument doc)
{
	dlgMain->AddMessage("OnUpdateObjectList");
}

//---------------------------------------------------------------------------
//  OnMaterialModified
//    マテリアルのパラメータ変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnMaterialModified(MQDocument doc)
{
	dlgMain->AddMessage("OnMaterialModified");
}

//---------------------------------------------------------------------------
//  OnUpdateMaterialList
//    カレントマテリアルの変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnUpdateMaterialList(MQDocument doc)
{
	dlgMain->AddMessage("OnUpdateMaterialList");
}

//---------------------------------------------------------------------------
//  OnUpdateScene
//    シーン情報の変更時
//---------------------------------------------------------------------------
void StationSpyPlugin::OnUpdateScene(MQDocument doc, MQScene scene)
{
	if (!m_bOnUpdateScene)
		return;

	dlgMain->AddMessage("OnUpdateScene");
}

//---------------------------------------------------------------------------
//  ExecuteCallback
//    コールバックに対する実装部
//---------------------------------------------------------------------------
bool StationSpyPlugin::ExecuteCallback(MQDocument doc, void *option)
{
	CallbackInfo *info = (CallbackInfo*)option;
	return ((*this).*info->proc)(doc);
}

// コールバックの呼び出し
void StationSpyPlugin::Execute(ExecuteCallbackProc proc)
{
	CallbackInfo info;
	info.proc = proc;
	BeginCallback(&info);
}

// プラグインのベースクラスを返す
MQBasePlugin *GetPluginClass()
{
	return &plugin;
}

